// Copyright © 22016 Université Paris-Sud, Written by Lénaïc Bagnères, lenaic.bagneres@u-psud.fr

// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include <iostream>
#include <vector>
#include <string>
#include <algorithm>
#include <limits>

#include "expression.hpp"
#include "factory.hpp"


int main()
{
	std::vector<std::string> lines =
	{
		"1",
		"x",
		"1 2 +",
		"1 2 + 4 * 3 +",
		"x 7 +",
		"y x - 7 + 2 /"
	};
	
	factory_t f;
	
	f.add
	(
		[](std::string const & string) -> bool { return hopp::is_integer(string); },
		[](std::string const & string) -> expression_t { return constant_t(std::stoi(string)); }
	);
	
	f.add
	(
		[](std::string const & string) -> bool
		{
			return std::find_if(string.begin(), string.end(), [](char const c) { return std::isalpha(c) == false; }) == string.end();
		},
		[](std::string const & string) -> expression_t { return variable_t(string, 0); }
	);
	
	f.add
	(
		[](std::string const & string) -> bool
		{
			return (string.size() == 1 && (string[0] == '+' || string[0] == '-' || string[0] == '*' || string[0] == '/'));
		},
		[](std::string const & string, expression_t && a, expression_t && b) -> expression_t
		{
			return operator_t(string[0], std::move(a), std::move(b));
		}
	);
	
	for (auto const & line : lines)
	{
		std::cout << line << std::endl;
		auto const expression = f.make(line);
		std::cout << "Expression = "; expression.display(); std::cout << std::endl;
		std::cout << "Value      = " << expression.eval() << std::endl;
		std::cout << std::endl;
	}
	
	return 0;
}
