// Copyright © 2016 Université Paris-Sud, Written by Lénaïc Bagnères, lenaic.bagneres@u-psud.fr

// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.


#ifndef FACTORY_HPP
#define FACTORY_HPP

#include <iostream>
#include <string>
#include <tuple>
#include <sstream>

#include "expression.hpp"


class factory_t
{
public:
	
	using fct_test_t = std::function<bool (std::string const & string)>;
	
	using fct_make_t = std::function<expression_t (std::string const & string)>;
	
	using fct_make_2_t = std::function<expression_t (std::string const & string, expression_t && a, expression_t && b)>;
	
private:
	
	std::vector<std::tuple<fct_test_t, bool, fct_make_2_t>> m_fcts;
	
public:
	
	void add(fct_test_t const & fct_test, fct_make_t const & fct_make)
	{
		m_fcts.emplace_back
		(
			fct_test,
			false,
			[fct_make](std::string const & string, expression_t &&, expression_t &&) -> expression_t
			{
				return fct_make(string);
			}
		);
	}
	
	void add(fct_test_t const & fct_test, fct_make_2_t const & fct_make)
	{
		m_fcts.emplace_back(fct_test, true, fct_make);
	}
	
	expression_t make(std::string const & string)
	{
		std::istringstream ss(string);
		
		std::string word;
		
		expression_t tmp_a;
		expression_t tmp_b;
		
		while (ss >> word)
		{
			expression_t tmp;
			
			for (auto const & fct_test_and_fct_make : m_fcts)
			{
				auto const & fct_test = std::get<0>(fct_test_and_fct_make);
				auto const & need_args = std::get<1>(fct_test_and_fct_make);
				auto const & fct_make = std::get<2>(fct_test_and_fct_make);
				
				if (fct_test(word))
				{
					if (need_args == false)
					{
						tmp = fct_make(word, expression_t(), expression_t());
					}
					else
					{
						tmp = fct_make(word, tmp_a.release(), tmp_b.release());
					}
					break;
				}
			}
			
			if (tmp_a.is_null())
			{
				tmp_a = std::move(tmp);
			}
			else if (tmp_b.is_null())
			{
				tmp_b = std::move(tmp);
			}
		}
		
		return tmp_a;
	}
};

#endif
